from datetime import datetime, timedelta
from json import dumps, loads

from panther_detection_helpers.caching import get_string_set, put_string_set

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_lookuptable_helpers import LookupTableMatches
from pypanther.helpers.panther_oss_helpers import km_between_ipinfo_loc, resolve_timestamp_string

standard_impossible_travel_login_tests: list[RuleTest] = [
    RuleTest(
        name="CloudTrail not ConsoleLogin",
        expected_result=False,
        log={"eventType": "logout", "p_log_type": "AWS.CloudTrail"},
    ),
    RuleTest(
        name="CloudTrail ConsoleLogin no history",
        expected_result=False,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(object_name="get_string_set", return_value=""),
        ],
        log={
            "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2023-05-26 20:14:51",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_event_time": "2023-05-26 20:14:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "sourceIPAddress": {
                        "city": "Auckland",
                        "country": "NZ",
                        "lat": "-36.84853",
                        "lng": "174.76349",
                        "p_match": "12.12.12.12",
                        "postal_code": "1010",
                        "region": "Auckland",
                        "region_code": "AUK",
                        "timezone": "Pacific/Auckland",
                    },
                },
            },
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2023-05-26 20:19:14.002",
            "p_source_label": "LogSource Name",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "12.12.12.12",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "signin.aws.amazon.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
        },
    ),
    RuleTest(
        name="CloudTrail ConsoleLogin with history",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n {\n  "p_event_time": "2023-05-26 18:14:51",\n  "city": "New York City",\n  "country": "US",\n  "lat": "40.71427",\n  "lng": "-74.00597",\n  "postal_code": "10004",\n  "region": "New York",\n  "region_code": "NY",\n  "timezone": "America/New_York"\n }\n]',
            ),
        ],
        log={
            "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2023-05-26 20:14:51",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_event_time": "2023-05-26 20:14:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "sourceIPAddress": {
                        "city": "Auckland",
                        "country": "NZ",
                        "lat": "-36.84853",
                        "lng": "174.76349",
                        "p_match": "12.12.12.12",
                        "postal_code": "1010",
                        "region": "Auckland",
                        "region_code": "AUK",
                        "timezone": "Pacific/Auckland",
                    },
                },
            },
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2023-05-26 20:19:14.002",
            "p_source_label": "LogSource Name",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "12.12.12.12",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "signin.aws.amazon.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
        },
    ),
    RuleTest(
        name="Okta Not sign-in",
        expected_result=False,
        log={"eventType": "logout", "p_log_type": "Okta.SystemLog"},
    ),
    RuleTest(
        name="Okta sign-in with history and impossible travel",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n {\n  "p_event_time": "2023-05-26 18:14:51",\n  "city": "New York City",\n  "country": "US",\n  "lat": "40.71427",\n  "lng": "-74.00597",\n  "postal_code": "10004",\n  "region": "New York",\n  "region_code": "NY",\n  "timezone": "America/New_York"\n }\n]',
            ),
        ],
        log={
            "actor": {
                "alternateId": "homer.simpson@company.com",
                "displayName": "Homer Simpson",
                "id": "00uwuwuwuwuwuwuwuwuw",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idx1234"},
            "client": {
                "device": "Computer",
                "ipAddress": "12.12.12.12",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": {}},
            "device": {},
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "p_event_time": "2023-05-26 20:18:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "client.ipAddress": {
                        "city": "Auckland",
                        "country": "NZ",
                        "lat": "-36.84853",
                        "lng": "174.76349",
                        "p_match": "12.12.12.12",
                        "postal_code": "1010",
                        "region": "Auckland",
                        "region_code": "AUK",
                        "timezone": "Pacific/Auckland",
                    },
                },
            },
            "p_log_type": "Okta.SystemLog",
            "p_source_label": "Okta Logs",
            "p_parse_time": "2023-05-26 20:22:51.888",
            "published": "2023-05-26 20:18:51.888",
            "request": {"ipChain": []},
            "securityContext": {},
            "severity": "INFO",
            "target": [],
            "transaction": {},
            "uuid": "79999999-ffff-eeee-bbbb-222222222222",
            "version": "0",
        },
    ),
    RuleTest(
        name="Okta sign-in with history and impossible travel, Apple Private Relay",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n {\n  "p_event_time": "2023-05-26 18:14:51",\n  "city": "New York City",\n  "country": "US",\n  "lat": "40.71427",\n  "lng": "-74.00597",\n  "postal_code": "10004",\n  "region": "New York",\n  "region_code": "NY",\n  "timezone": "America/New_York"\n }\n]',
            ),
        ],
        log={
            "actor": {
                "alternateId": "homer.simpson@company.com",
                "displayName": "Homer Simpson",
                "id": "00uwuwuwuwuwuwuwuwuw",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idx1234"},
            "client": {
                "device": "Computer",
                "ipAddress": "12.12.12.12",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": {}},
            "device": {},
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "p_event_time": "2023-05-26 20:18:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "client.ipAddress": {
                        "city": "Los Angeles",
                        "country": "US",
                        "lat": "34.05223",
                        "lng": "-118.24368",
                        "p_match": "12.12.12.12",
                        "postal_code": "90009",
                        "region": "California",
                        "region_code": "CA",
                        "timezone": "America/Los_Angeles",
                    },
                },
                "ipinfo_privacy": {
                    "client.ipAddress": {
                        "hosting": True,
                        "p_match": "12.12.12.12",
                        "proxy": False,
                        "relay": True,
                        "service": "Apple Private Relay",
                        "tor": False,
                        "vpn": False,
                    },
                },
            },
            "p_log_type": "Okta.SystemLog",
            "p_source_label": "Okta Logs",
            "p_parse_time": "2023-05-26 20:22:51.888",
            "published": "2023-05-26 20:18:51.888",
            "request": {"ipChain": []},
            "securityContext": {},
            "severity": "INFO",
            "target": [],
            "transaction": {},
            "uuid": "79999999-ffff-eeee-bbbb-222222222222",
            "version": "0",
        },
    ),
    RuleTest(
        name="Okta sign-in with history and impossible travel, VPN with service",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n {\n  "p_event_time": "2023-05-26 18:14:51",\n  "city": "New York City",\n  "country": "US",\n  "lat": "40.71427",\n  "lng": "-74.00597",\n  "postal_code": "10004",\n  "region": "New York",\n  "region_code": "NY",\n  "timezone": "America/New_York"\n }\n]',
            ),
        ],
        log={
            "actor": {
                "alternateId": "homer.simpson@company.com",
                "displayName": "Homer Simpson",
                "id": "00uwuwuwuwuwuwuwuwuw",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idx1234"},
            "client": {
                "device": "Computer",
                "ipAddress": "12.12.12.12",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": {}},
            "device": {},
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "p_event_time": "2023-05-26 20:18:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "client.ipAddress": {
                        "city": "Los Angeles",
                        "country": "US",
                        "lat": "34.05223",
                        "lng": "-118.24368",
                        "p_match": "12.12.12.12",
                        "postal_code": "90009",
                        "region": "California",
                        "region_code": "CA",
                        "timezone": "America/Los_Angeles",
                    },
                },
                "ipinfo_privacy": {
                    "client.ipAddress": {
                        "hosting": False,
                        "p_match": "12.12.12.12",
                        "proxy": False,
                        "relay": False,
                        "service": "Private Internet Access",
                        "tor": False,
                        "vpn": True,
                    },
                },
            },
            "p_log_type": "Okta.SystemLog",
            "p_source_label": "Okta Logs",
            "p_parse_time": "2023-05-26 20:22:51.888",
            "published": "2023-05-26 20:18:51.888",
            "request": {"ipChain": []},
            "securityContext": {},
            "severity": "INFO",
            "target": [],
            "transaction": {},
            "uuid": "79999999-ffff-eeee-bbbb-222222222222",
            "version": "0",
        },
    ),
    RuleTest(
        name="Okta sign-in with history and impossible travel, VPN with no service",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n {\n  "p_event_time": "2023-05-26 18:14:51",\n  "city": "New York City",\n  "country": "US",\n  "lat": "40.71427",\n  "lng": "-74.00597",\n  "postal_code": "10004",\n  "region": "New York",\n  "region_code": "NY",\n  "timezone": "America/New_York"\n }\n]',
            ),
        ],
        log={
            "actor": {
                "alternateId": "homer.simpson@company.com",
                "displayName": "Homer Simpson",
                "id": "00uwuwuwuwuwuwuwuwuw",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idx1234"},
            "client": {
                "device": "Computer",
                "ipAddress": "12.12.12.12",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": {}},
            "device": {},
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "p_event_time": "2023-05-26 20:18:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "client.ipAddress": {
                        "city": "Los Angeles",
                        "country": "US",
                        "lat": "34.05223",
                        "lng": "-118.24368",
                        "p_match": "12.12.12.12",
                        "postal_code": "90009",
                        "region": "California",
                        "region_code": "CA",
                        "timezone": "America/Los_Angeles",
                    },
                },
                "ipinfo_privacy": {
                    "client.ipAddress": {
                        "hosting": False,
                        "proxy": False,
                        "relay": False,
                        "service": "",
                        "tor": False,
                        "vpn": True,
                    },
                },
            },
            "p_log_type": "Okta.SystemLog",
            "p_source_label": "Okta Logs",
            "p_parse_time": "2023-05-26 20:22:51.888",
            "published": "2023-05-26 20:18:51.888",
            "request": {"ipChain": []},
            "securityContext": {},
            "severity": "INFO",
            "target": [],
            "transaction": {},
            "uuid": "79999999-ffff-eeee-bbbb-222222222222",
            "version": "0",
        },
    ),
    RuleTest(
        name="Short Distances and Short Timedeltas",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n  {\n    "city": "Los Angeles",\n    "country": "US",\n    "lat": "34.05223",\n    "lng": "-118.24368",\n    "p_event_time": "2023-06-12T22:23:51.964000",\n    "postal_code": "90009",\n    "region": "California",\n    "region_code": "CA",\n    "timezone": "America/Los_Angeles"\n  }\n]',
            ),
        ],
        log={
            "actor": {
                "alternateId": "homer.simpson@company.com",
                "displayName": "Homer Simpson",
                "id": "00uwuwuwuwuwuwuwuwuw",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idx1234"},
            "client": {
                "device": "Computer",
                "ipAddress": "12.12.12.11",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": {}},
            "device": {},
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "p_event_time": "2023-06-12T22:26:01.951000",
            "p_enrichment": {
                "ipinfo_location": {
                    "client.ipAddress": {
                        "p_match": "12.12.12.11",
                        "city": "Anaheim",
                        "country": "US",
                        "lat": "33.8085",
                        "lng": "-117.9228",
                        "p_event_time": "2023-06-12T22:26:01.951000",
                        "postal_code": "92802",
                        "region": "California",
                        "region_code": "CA",
                        "timezone": "America/Los_Angeles",
                    },
                },
            },
            "p_log_type": "Okta.SystemLog",
            "p_source_label": "Okta Logs",
            "p_parse_time": "2023-06-12T22:29:01.951000",
            "published": "2023-06-12 22:26:01.951000",
            "request": {"ipChain": []},
            "securityContext": {},
            "severity": "INFO",
            "target": [],
            "transaction": {},
            "uuid": "79999999-ffff-eeee-bbbb-222222222222",
            "version": "0",
        },
    ),
    RuleTest(
        name="Asana ImpossibleTravel",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n {\n  "p_event_time": "2023-06-12T21:26:01.951000",\n  "city": "New York City",\n  "country": "US",\n  "lat": "40.71427",\n  "lng": "-74.00597",\n  "postal_code": "10004",\n  "region": "New York",\n  "region_code": "NY",\n  "timezone": "America/New_York"\n }\n]',
            ),
        ],
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpsons@simpsons.com",
                "gid": "1234567890",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "1.2.3.4",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
            },
            "created_at": "2023-05-26 18:24:51.413",
            "details": {"method": ["SAML"]},
            "event_category": "logins",
            "event_type": "user_login_succeeded",
            "gid": "123456789",
            "resource": {
                "email": "homer.simpsons@simpsons.com",
                "gid": "1234567890",
                "name": "Homer Simpson",
                "resource_type": "user",
            },
            "p_log_type": "Asana.Audit",
            "p_source_label": "Asana Logs",
            "p_enrichment": {
                "ipinfo_privacy": {"context.client_ip_address": {"p_match": "1.2.3.4"}},
                "ipinfo_location": {
                    "context.client_ip_address": {
                        "p_match": "1.2.3.4",
                        "city": "Anaheim",
                        "country": "US",
                        "lat": "33.8085",
                        "lng": "-117.9228",
                        "p_event_time": "2023-06-12T22:26:01.951000",
                        "postal_code": "92802",
                        "region": "California",
                        "region_code": "CA",
                        "timezone": "America/Los_Angeles",
                    },
                },
            },
            "p_event_time": "2023-06-12T22:26:01.951000",
        },
    ),
    RuleTest(
        name="Notion Impossible Travel",
        expected_result=True,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(
                object_name="get_string_set",
                return_value='[\n  {\n    "p_event_time": "2023-10-03T18:26:01.951000",\n    "source_ip": "192.168.100.100",\n    "city": "Minas Tirith",\n    "country": "Gondor",\n    "lat": "0.00000",\n    "lng": "0.00000",\n    "p_match": "192.168.100.100",\n    "postal_code": "55555",\n    "region": "Pellenor",\n    "region_code": "PL",\n    "timezone": "Middle Earth/Pellenor"\n  }\n]',
            ),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "saml"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "timestamp": "2023-10-03T19:02:28.044000Z",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "100.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "p_match": "192.168.100.100",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-10-03T19:02:28.044000Z",
            "p_log_type": "Notion.AuditLogs",
            "p_source_label": "Notion-Panther-Labs",
        },
    ),
    RuleTest(
        name="First hit from VPN should not fail",
        expected_result=False,
        mocks=[
            RuleMock(object_name="put_string_set", return_value=""),
            RuleMock(object_name="get_string_set", return_value=""),
        ],
        log={
            "actor": {
                "alternateId": "homer.simpson@company.com",
                "displayName": "Homer Simpson",
                "id": "00uwuwuwuwuwuwuwuwuw",
                "type": "User",
            },
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "idx1234"},
            "client": {
                "device": "Computer",
                "ipAddress": "12.12.12.12",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {"debugData": {}},
            "device": {},
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_success",
            "outcome": {"result": "SUCCESS"},
            "p_event_time": "2023-05-26 20:18:51",
            "p_enrichment": {
                "ipinfo_location": {
                    "client.ipAddress": {
                        "city": "Los Angeles",
                        "country": "US",
                        "lat": "34.05223",
                        "lng": "-118.24368",
                        "p_match": "12.12.12.12",
                        "postal_code": "90009",
                        "region": "California",
                        "region_code": "CA",
                        "timezone": "America/Los_Angeles",
                    },
                },
                "ipinfo_privacy": {
                    "client.ipAddress": {
                        "hosting": True,
                        "p_match": "12.12.12.12",
                        "proxy": False,
                        "relay": True,
                        "service": "Apple Private Relay",
                        "tor": False,
                        "vpn": False,
                    },
                },
            },
            "p_log_type": "Okta.SystemLog",
            "p_source_label": "Okta Logs",
            "p_parse_time": "2023-05-26 20:22:51.888",
            "published": "2023-05-26 20:18:51.888",
            "request": {"ipChain": []},
            "securityContext": {},
            "severity": "INFO",
            "target": [],
            "transaction": {},
            "uuid": "79999999-ffff-eeee-bbbb-222222222222",
            "version": "0",
        },
    ),
]


@panther_managed
class StandardImpossibleTravelLogin(Rule):
    id = "Standard.ImpossibleTravel.Login-prototype"
    display_name = "Impossible Travel for Login Action"
    log_types = [LogType.ASANA_AUDIT, LogType.AWS_CLOUDTRAIL, LogType.NOTION_AUDIT_LOGS, LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "Initial Access:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.HIGH
    default_description = "A user has subsequent logins from two geographic locations that are very far apart"
    default_runbook = "Reach out to the user if needed to validate the activity, then lock the account.\nIf the user responds that the geolocation on the new location is incorrect, you can directly report the inaccuracy via  https://ipinfo.io/corrections\n"
    default_reference = "https://expertinsights.com/insights/what-are-impossible-travel-logins/#:~:text=An%20impossible%20travel%20login%20is,of%20the%20logins%20is%20fraudulent"
    summary_attributes = ["p_any_usernames", "p_any_ip_addresses", "p_any_domain_names"]
    tests = standard_impossible_travel_login_tests

    def gen_key(self, event):
        """
        gen_key uses the data_model for the logtype to cache
        an entry that is specific to the Log Source ID

        The data_model needs to answer to "actor_user"
        """
        rule_name = deep_get(event, "p_source_label")
        actor = event.udm("actor_user")
        if None in [rule_name, actor]:
            return None
        return f"{rule_name.replace(' ', '')}..{actor}"

    def rule(self, event):
        # too-many-return-statements due to error checking
        # pylint: disable=global-statement,too-many-return-statements,too-complex
        self.EVENT_CITY_TRACKING = {}
        self.CACHE_KEY = ""
        self.IS_VPN = False
        self.IS_PRIVATE_RELAY = False
        # Only evaluate successful logins
        if event.udm("event_type") != event_type.SUCCESSFUL_LOGIN:
            return False
        p_event_datetime = resolve_timestamp_string(deep_get(event, "p_event_time"))
        if p_event_datetime is None:
            # we couldn't go from p_event_time to a datetime object
            # we need to do this in order to make later time comparisons generic
            return False
        new_login_stats = {"p_event_time": p_event_datetime.isoformat(), "source_ip": event.udm("source_ip")}
        #
        src_ip_enrichments = LookupTableMatches().p_matches(event, event.udm("source_ip"))
        # stuff everything from ipinfo_location into the new_login_stats
        # new_login_stats is the value that we will cache for this key
        ipinfo_location = deep_get(src_ip_enrichments, "ipinfo_location")
        if ipinfo_location is None:
            return False
        new_login_stats.update(ipinfo_location)
        # Bail out if we have a None value in set as it causes false positives
        if None in new_login_stats.values():
            return False
        ## Check for VPN or Private Relay
        ipinfo_privacy = deep_get(src_ip_enrichments, "ipinfo_privacy")
        if ipinfo_privacy is not None:
            ###  Do VPN/private relay
            self.IS_PRIVATE_RELAY = all(
                [
                    deep_get(ipinfo_privacy, "relay", default=False),
                    deep_get(ipinfo_privacy, "service", default="") == "Apple Private Relay",
                ],
            )
            # We've found that some places, like WeWork locations,
            #   have the VPN attribute set to true, but do not have a
            #   service name entry.
            # We have noticed VPN connections with commercial VPN
            #   offerings have the VPN attribute set to true, and
            #   do have a service name entry
            self.IS_VPN = all(
                [deep_get(ipinfo_privacy, "vpn", default=False), deep_get(ipinfo_privacy, "service", default="") != ""],
            )
        if self.IS_VPN or self.IS_PRIVATE_RELAY:
            new_login_stats.update(
                {
                    "is_vpn": f"{self.IS_VPN}",
                    "is_apple_priv_relay": f"{self.IS_PRIVATE_RELAY}",
                    "service_name": f"{deep_get(ipinfo_privacy, 'service', default='<NO_SERVICE>')}",
                    "NOTE": "APPLE PRIVATE RELAY AND VPN LOGINS ARE NOT CACHED FOR COMPARISON",
                },
            )
        # Generate a unique cache key for each user per log type
        self.CACHE_KEY = self.gen_key(event)
        if not self.CACHE_KEY:
            # We can't save without a cache key
            return False
        # Retrieve the prior login info from the cache, if any
        last_login = get_string_set(self.CACHE_KEY)
        # If we haven't seen this user login in the past 1 day,
        # store this login for future use and don't alert
        if not last_login:
            if not (self.IS_PRIVATE_RELAY or self.IS_VPN):
                put_string_set(
                    key=self.CACHE_KEY,
                    val=[dumps(new_login_stats)],
                    epoch_seconds=int((datetime.utcnow() + timedelta(days=1)).timestamp()),
                )
            return False
        # Load the last login from the cache into an object we can compare
        # str check is in place for unit test mocking
        if isinstance(last_login, str):
            tmp_last_login = loads(last_login)
            last_login = []
            for l_l in tmp_last_login:
                last_login.append(dumps(l_l))
        last_login_stats = loads(last_login.pop())
        distance = km_between_ipinfo_loc(last_login_stats, new_login_stats)
        old_time = resolve_timestamp_string(deep_get(last_login_stats, "p_event_time"))
        new_time = resolve_timestamp_string(deep_get(new_login_stats, "p_event_time"))
        time_delta = (new_time - old_time).total_seconds() / 3600  # seconds in an hour
        # Don't let time_delta be 0 (divide by zero error below)
        time_delta = time_delta or 0.0001
        # Calculate speed in Kilometers / Hour
        speed = distance / time_delta
        # Calculation is complete, write the current login to the cache
        # Only if non-VPN non-relay!
        if not self.IS_PRIVATE_RELAY and (not self.IS_VPN):
            put_string_set(
                key=self.CACHE_KEY,
                val=[dumps(new_login_stats)],
                epoch_seconds=int((datetime.utcnow() + timedelta(days=1)).timestamp()),
            )
        self.EVENT_CITY_TRACKING["previous"] = last_login_stats
        self.EVENT_CITY_TRACKING["current"] = new_login_stats
        self.EVENT_CITY_TRACKING["speed"] = int(speed)
        self.EVENT_CITY_TRACKING["speed_units"] = "km/h"
        self.EVENT_CITY_TRACKING["distance"] = int(distance)
        self.EVENT_CITY_TRACKING["distance_units"] = "km"
        return speed > 900  # Boeing 747 cruising speed

    def title(self, event):
        #
        log_source = deep_get(event, "p_source_label", default="<NO_SOURCE_LABEL>")
        old_city = deep_get(self.EVENT_CITY_TRACKING, "previous", "city", default="<NO_PREV_CITY>")
        new_city = deep_get(self.EVENT_CITY_TRACKING, "current", "city", default="<NO_PREV_CITY>")
        speed = deep_get(self.EVENT_CITY_TRACKING, "speed", default="<NO_SPEED>")
        distance = deep_get(self.EVENT_CITY_TRACKING, "distance", default="<NO_DISTANCE>")
        return f"Impossible Travel: [{event.udm('actor_user')}] in [{log_source}] went [{speed}] km/h for [{distance}] km between [{old_city}] and [{new_city}]"

    def dedup(self, event):  # pylint: disable=W0613
        return self.CACHE_KEY

    def alert_context(self, event):
        context = {"actor_user": event.udm("actor_user")}
        context.update(self.EVENT_CITY_TRACKING)
        return context

    def severity(self, _):
        if self.IS_VPN or self.IS_PRIVATE_RELAY:
            return "INFO"
        # time = distance/speed
        distance = deep_get(self.EVENT_CITY_TRACKING, "distance", default=None)
        speed = deep_get(self.EVENT_CITY_TRACKING, "speed", default=None)
        if speed and distance:
            time = distance / speed
            # time of 0.1666 is 10 minutes
            if time < 0.1666 and distance < 50:
                # This is likely a GEOIP inaccuracy
                return "LOW"
        return "HIGH"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

carbon_black_audit_data_forwarder_stopped_tests: list[RuleTest] = [
    RuleTest(
        name="Endpoint event forwarder disabled",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": 'Updated Config: {"id":"b6ab1fb9-61f6-11ee-9e9b-5655adf4bf96","org_key":"A1234567","name":"endpoint event","enabled":false,"s3_bucket_name":"carbonblackbucket","s3_prefix":"endpoint","type":"endpoint.event","create_time":"2023-10-03T14:11:14Z","update_time":"2023-11-14T19:16:43Z"}\n',
            "eventId": "58bef441832211ee83ef1721d866b8d6",
            "eventTime": "2023-11-14 19:16:43.123000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "bob.ross@acme.com",
            "requestUrl": "/data_forwarder/v2/orgs/A1234567/configs/b6ab1fb9-61f6-11ee-9e9b-5655adf4bf96",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Endpoint event forwarder updated",
        expected_result=False,
        log={
            "clientIp": "12.34.56.78",
            "description": 'Updated Config: {"id":"b6ab1fb9-61f6-11ee-9e9b-5655adf4bf96","org_key":"A1234567","name":"endpoint event","enabled":true,"s3_bucket_name":"carbonblackbucket","s3_prefix":"endpoint","type":"endpoint.event","create_time":"2023-10-03T14:11:14Z","update_time":"2023-11-14T19:16:43Z"}\n',
            "eventId": "58bef441832211ee83ef1721d866b8d6",
            "eventTime": "2023-11-14 19:16:43.123000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "bob.ross@acme.com",
            "requestUrl": "/data_forwarder/v2/orgs/A1234567/configs/b6ab1fb9-61f6-11ee-9e9b-5655adf4bf96",
            "verbose": False,
        },
    ),
]


@panther_managed
class CarbonBlackAuditDataForwarderStopped(Rule):
    id = "CarbonBlack.Audit.Data.Forwarder.Stopped-prototype"
    log_types = [LogType.CARBON_BLACK_AUDIT]
    default_description = "Detects when a user disables or deletes a Data Forwarder."
    display_name = "Carbon Black Data Forwarder Stopped"
    default_severity = Severity.HIGH
    tags = ["Defense Evasion", "Impair Defenses", "Disable or Modify Cloud Logs"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008"]}
    default_reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-E8D33F72-BABB-4157-A908-D8BBDB5AF349.html"
    tests = carbon_black_audit_data_forwarder_stopped_tests
    ACTION = ""

    def rule(self, event):
        if not event.get("requestUrl", "").startswith("/data_forwarder/"):
            return False
        desc = event.get("description", "")
        if desc.startswith("Deleted Config: "):
            self.ACTION = "Deleted"
            return True
        if desc.startswith("Updated Config: ") and '"enabled":false' in desc:
            self.ACTION = "Disabled"
            return True
        return False

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        return f"{user} [{ip_addr}] {self.ACTION} Data Forwarder"

    def description(self, event):
        user = event.get("loginName")
        ip_addr = event.get("clientIp")
        desc = event.get("description")
        return f"{user} [{ip_addr}] {desc}"



from pypanther.base import DataModel, DataModelMapping, LogType


def get_dns_query(event):
    # Strip trailing period.
    # Domain Names from Cisco Umbrella end with a trailing period, such as google.com.
    domain = event.get("domain")
    if domain:
        domain = domain.rstrip(".")
    return domain


class StandardCiscoUmbrellaDNS(DataModel):
    id: str = "Standard.CiscoUmbrella.DNS"
    display_name: str = "Cisco Umbrella DNS"
    enabled: bool = True
    log_types: list[str] = [LogType.CISCO_UMBRELLA_DNS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="source_ip", path="internalIp"),
        DataModelMapping(name="source_port", path="srcPort"),
        DataModelMapping(name="dns_query", method=get_dns_query),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_cloudflare_helpers import cloudflare_http_alert_context

cloudflare_http_request_bot_high_volume_tests: list[RuleTest] = [
    RuleTest(
        name="Likely Human",
        expected_result=False,
        log={
            "BotScore": 99,
            "CacheCacheStatus": "miss",
            "CacheResponseBytes": 76931,
            "CacheResponseStatus": 404,
            "CacheTieredFill": False,
            "ClientASN": 63949,
            "ClientCountry": "gb",
            "ClientDeviceType": "desktop",
            "ClientIP": "142.93.204.250",
            "ClientIPClass": "noRecord",
            "ClientRequestBytes": 2407,
            "ClientRequestHost": "example.com",
            "ClientRequestMethod": "GET",
            "ClientRequestPath": "",
            "ClientRequestProtocol": "HTTP/1.1",
            "ClientRequestReferer": "https://example.com/",
            "ClientRequestURI": "",
            "ClientRequestUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.97 Safari/537.36",
            "ClientSSLProtocol": "TLSv1.3",
            "ClientSrcPort": 28057,
            "ClientXRequestedWith": "",
            "EdgeColoCode": "LHR",
            "EdgeColoID": 373,
            "EdgeEndTimestamp": "2022-05-07 18:53:13",
            "EdgePathingOp": "wl",
            "EdgePathingSrc": "macro",
            "EdgePathingStatus": "nr",
            "EdgeRateLimitAction": "",
            "EdgeRateLimitID": "0",
            "EdgeRequestHost": "example.com",
            "EdgeResponseBytes": 17826,
            "EdgeResponseCompressionRatio": 4.55,
            "EdgeResponseContentType": "text/html",
            "EdgeResponseStatus": 404,
            "EdgeServerIP": "",
            "EdgeStartTimestamp": "2022-05-07 18:53:12",
            "OriginIP": "",
            "OriginResponseBytes": 0,
            "OriginResponseStatus": 0,
            "OriginResponseTime": 0,
            "OriginSSLProtocol": "unknown",
            "ParentRayID": "00",
            "RayID": "707c283ab88274cd",
            "SecurityLevel": "med",
            "WAFAction": "unknown",
            "WAFFlags": "0",
            "WAFMatchedVar": "",
            "WAFProfile": "unknown",
            "WAFRuleID": "",
            "WAFRuleMessage": "",
            "WorkerCPUTime": 0,
            "WorkerStatus": "unknown",
            "WorkerSubrequest": False,
            "WorkerSubrequestCount": 0,
            "ZoneID": 526503649,
            "p_any_domain_names": ["https://example.com/", "example.com"],
            "p_any_ip_addresses": ["142.93.204.250"],
            "p_any_trace_ids": ["00", "707c283ab88274cd"],
            "p_event_time": "2022-05-07 18:53:12",
            "p_log_type": "Cloudflare.HttpRequest",
            "p_parse_time": "2022-05-07 18:54:31.922",
            "p_row_id": "a6e3965df054cfcdbdccf3ec10a134",
            "p_source_id": "2b9fc5ae-9cab-4715-8683-9bfbdb15a313",
            "p_source_label": "Cloudflare",
        },
    ),
    RuleTest(
        name="Likely Automated",
        expected_result=True,
        log={
            "BotScore": 29,
            "CacheCacheStatus": "miss",
            "CacheResponseBytes": 76931,
            "CacheResponseStatus": 404,
            "CacheTieredFill": False,
            "ClientASN": 63949,
            "ClientCountry": "gb",
            "ClientDeviceType": "desktop",
            "ClientIP": "142.93.204.250",
            "ClientIPClass": "noRecord",
            "ClientRequestBytes": 2407,
            "ClientRequestHost": "example.com",
            "ClientRequestMethod": "GET",
            "ClientRequestPath": "",
            "ClientRequestProtocol": "HTTP/1.1",
            "ClientRequestReferer": "https://example.com/",
            "ClientRequestURI": "",
            "ClientRequestUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.97 Safari/537.36",
            "ClientSSLProtocol": "TLSv1.3",
            "ClientSrcPort": 28057,
            "ClientXRequestedWith": "",
            "EdgeColoCode": "LHR",
            "EdgeColoID": 373,
            "EdgeEndTimestamp": "2022-05-07 18:53:13",
            "EdgePathingOp": "wl",
            "EdgePathingSrc": "macro",
            "EdgePathingStatus": "nr",
            "EdgeRateLimitAction": "",
            "EdgeRateLimitID": "0",
            "EdgeRequestHost": "example.com",
            "EdgeResponseBytes": 17826,
            "EdgeResponseCompressionRatio": 4.55,
            "EdgeResponseContentType": "text/html",
            "EdgeResponseStatus": 404,
            "EdgeServerIP": "",
            "EdgeStartTimestamp": "2022-05-07 18:53:12",
            "OriginIP": "",
            "OriginResponseBytes": 0,
            "OriginResponseStatus": 0,
            "OriginResponseTime": 0,
            "OriginSSLProtocol": "unknown",
            "ParentRayID": "00",
            "RayID": "707c283ab88274cd",
            "SecurityLevel": "med",
            "WAFAction": "unknown",
            "WAFFlags": "0",
            "WAFMatchedVar": "",
            "WAFProfile": "unknown",
            "WAFRuleID": "",
            "WAFRuleMessage": "",
            "WorkerCPUTime": 0,
            "WorkerStatus": "unknown",
            "WorkerSubrequest": False,
            "WorkerSubrequestCount": 0,
            "ZoneID": 526503649,
            "p_any_domain_names": ["https://example.com/", "example.com"],
            "p_any_ip_addresses": ["142.93.204.250"],
            "p_any_trace_ids": ["00", "707c283ab88274cd"],
            "p_event_time": "2022-05-07 18:53:12",
            "p_log_type": "Cloudflare.HttpRequest",
            "p_parse_time": "2022-05-07 18:54:31.922",
            "p_row_id": "a6e3965df054cfcdbdccf3ec10a134",
            "p_source_id": "2b9fc5ae-9cab-4715-8683-9bfbdb15a313",
            "p_source_label": "Cloudflare",
        },
    ),
]


@panther_managed
class CloudflareHttpRequestBotHighVolume(Rule):
    id = "Cloudflare.HttpRequest.BotHighVolume-prototype"
    display_name = "Cloudflare Bot High Volume"
    enabled = False
    log_types = [LogType.CLOUDFLARE_HTTP_REQUEST]
    tags = ["Cloudflare"]
    default_severity = Severity.LOW
    default_description = "Monitors for bots making HTTP Requests at a rate higher than 2req/sec"
    default_runbook = "Inspect and monitor internet-facing services for potential outages"
    default_reference = "https://developers.cloudflare.com/waf/rate-limiting-rules/request-rate/"
    threshold = 7560
    summary_attributes = [
        "ClientIP",
        "ClientRequestUserAgent",
        "EdgeResponseContentType",
        "ClientCountry",
        "ClientRequestURI",
    ]
    tests = cloudflare_http_request_bot_high_volume_tests

    def rule(self, event):
        # Bot scores are [0, 99] where scores >0 && <30 indicating likely automated
        # https://developers.cloudflare.com/bots/concepts/bot-score/
        return all([event.get("BotScore", 100) <= 30, event.get("BotScore", 100) >= 1])

    def title(self, event):
        return f"Cloudflare: High Volume of Bot Requests - from [{event.get('ClientIP', '<NO_CLIENTIP>')}] to [{event.get('ClientRequestHost', '<NO_REQ_HOST>')}]"

    def dedup(self, event):
        return f"{event.get('ClientIP', '<NO_CLIENTIP>')}:{event.get('ClientRequestHost', '<NO_REQ_HOST>')}"

    def alert_context(self, event):
        return cloudflare_http_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_scheduled_jobs_tests: list[RuleTest] = [
    RuleTest(
        name="Crontab no args",
        expected_result=True,
        log={
            "argv": [],
            "cgroup_id": 4294967717,
            "code": "T4000I",
            "ei": 39,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/crontab",
            "pid": 18415,
            "ppid": 18413,
            "program": "crontab",
            "return_code": 0,
            "server_id": "e073ecab-6091-45da-83e4-80196e7bc659",
            "sid": "29a3d18c-2c05-453d-979a-2ed888a14788",
            "time": "2020-08-18T00:05:12.465Z",
            "uid": "83e88438-efbc-41a2-8135-b0157e0d14c0",
            "user": "panther",
        },
    ),
    RuleTest(
        name="Crontab Edit",
        expected_result=True,
        log={
            "argv": ["-e"],
            "cgroup_id": 4294967582,
            "code": "T4000I",
            "ei": 50,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/crontab",
            "pid": 9451,
            "ppid": 9217,
            "program": "crontab",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "af24d0b8-9767-4bd8-99ce-7a4449ee3eba",
            "time": "2020-08-17T18:54:32.273Z",
            "uid": "ad4a31d0-d739-4409-8f1c-cf573ed97a89",
            "user": "panther",
        },
    ),
    RuleTest(
        name="Crontab List",
        expected_result=False,
        log={
            "argv": ["-l"],
            "cgroup_id": 4294967582,
            "code": "T4000I",
            "ei": 37,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/crontab",
            "pid": 9330,
            "ppid": 9315,
            "program": "crontab",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "af24d0b8-9767-4bd8-99ce-7a4449ee3eba",
            "time": "2020-08-17T18:50:39.1Z",
            "uid": "6b463839-c641-43d3-ab97-3137ff9b09f8",
            "user": "panther",
        },
    ),
    RuleTest(
        name="Echo command",
        expected_result=False,
        log={
            "argv": [],
            "cgroup_id": 4294967537,
            "code": "T4000I",
            "ei": 15,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/echo",
            "pid": 7143,
            "ppid": 7115,
            "program": "echo",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
            "time": "2020-08-17T17:40:37.491Z",
            "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
            "user": "panther",
        },
    ),
]


@panther_managed
class TeleportScheduledJobs(Rule):
    id = "Teleport.ScheduledJobs-prototype"
    display_name = "Teleport Scheduled Jobs"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["SSH", "Execution:Scheduled Task/Job"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0002:T1053"]}
    default_description = "A user has manually edited the Linux crontab"
    threshold = 10
    dedup_period_minutes = 15
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Validate the user behavior and rotate the host if necessary."
    summary_attributes = ["event", "code", "user", "program", "path", "return_code", "login", "server_id", "sid"]
    tests = teleport_scheduled_jobs_tests

    def rule(self, event):
        # Filter the events
        if event.get("event") != "session.command":
            return False
        # Ignore list/read events
        if "-l" in event.get("argv", []):
            return False
        return event.get("program") == "crontab"

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] has modified scheduled jobson [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcpgke_kubernetes_cron_job_created_or_modified_tests: list[RuleTest] = [
    RuleTest(
        name="create",
        expected_result=True,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": True, "permission": "io.k8s.batch.v1.cronjobs.create"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    RuleTest(
        name="update",
        expected_result=True,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": True, "permission": "io.k8s.batch.v1.cronjobs.update"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    RuleTest(
        name="fail",
        expected_result=False,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": False, "permission": "cloudfunctions.functions.upsert"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
]


@panther_managed
class GCPGKEKubernetesCronJobCreatedOrModified(Rule):
    id = "GCP.GKE.Kubernetes.Cron.Job.Created.Or.Modified-prototype"
    display_name = "GCP GKE Kubernetes Cron Job Created Or Modified"
    default_description = "This detection monitor for any modifications or creations of a cron job in GKE. Attackers may create or modify an existing scheduled job in order to achieve cluster persistence."
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.MEDIUM
    default_reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    default_runbook = "Investigate a reason of creating or modifying a cron job in GKE. Create ticket if appropriate."
    reports = {"MITRE ATT&CK": ["T1053.003"]}
    tests = gcpgke_kubernetes_cron_job_created_or_modified_tests

    def rule(self, event):
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if (
                auth.get("permission") in ["io.k8s.batch.v1.cronjobs.create", "io.k8s.batch.v1.cronjobs.update"]
                and auth.get("granted") is True
            ):
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gcpk8s_ioc_activity_tests: list[RuleTest] = [
    RuleTest(
        name="triggers",
        expected_result=True,
        log={"operation": {"producer": "k8s.io"}, "p_enrichment": {"tor_exit_nodes": ["1.1.1.1"]}},
    ),
    RuleTest(
        name="ignore",
        expected_result=False,
        log={"operation": {"producer": "chrome"}, "p_enrichment": {"tor_exit_nodes": ["1.1.1.1"]}},
    ),
]


@panther_managed
class GCPK8sIOCActivity(Rule):
    id = "GCP.K8s.IOC.Activity-prototype"
    display_name = "GCP K8s IOCActivity"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Optional"]
    default_severity = Severity.MEDIUM
    default_description = (
        "This detection monitors for any kubernetes API Request originating from an Indicator of Compromise."
    )
    reports = {"MITRE ATT&CK": ["T1573.002"]}
    default_runbook = "Add IP address the request is originated from to banned addresses."
    default_reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    tests = gcpk8s_ioc_activity_tests

    def rule(self, event):
        if deep_get(event, "operation", "producer") == "k8s.io" and deep_get(event, "p_enrichment", "tor_exit_nodes"):
            return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        context["tor_exit_nodes"] = deep_get(event, "p_enrichment", "tor_exit_nodes")
        return context



"""Used to define global helpers and variables."""

"""Utility functions provided to policies and rules during execution."""

import json
import os
import re
from datetime import datetime
from ipaddress import ip_address
from math import atan2, cos, radians, sin, sqrt
from typing import Any, Dict, Optional, Sequence, Set, Union

import boto3
import requests
from dateutil import parser
from panther_detection_helpers import caching

_RESOURCE_TABLE = None  # boto3.Table resource, lazily constructed
FIPS_ENABLED = os.getenv("ENABLE_FIPS", "").lower() == "true"
FIPS_SUFFIX = "-fips." + os.getenv("AWS_REGION", "") + ".amazonaws.com"

# Auto Time Resolution Parameters
EPOCH_REGEX = r"([0-9]{9,12}(\.\d+)?)"
TIME_FORMATS = [
    "%Y-%m-%d %H:%M:%S",  # Panther p_event_time Timestamp
    "%Y-%m-%dT%H:%M:%SZ",  # AWS Timestamp
    "%Y-%m-%dT%H:%M:%S.%fZ",  # Panther Timestamp
    "%Y-%m-%dT%H:%M:%S*%f%z",
    "%Y %b %d %H:%M:%S.%f %Z",
    "%b %d %H:%M:%S %z %Y",
    "%d/%b/%Y:%H:%M:%S %z",
    "%b %d, %Y %I:%M:%S %p",
    "%b %d %Y %H:%M:%S",
    "%b %d %H:%M:%S %Y",
    "%b %d %H:%M:%S %z",
    "%b %d %H:%M:%S",
    "%Y-%m-%dT%H:%M:%S%z",
    "%Y-%m-%dT%H:%M:%S.%f%z",
    "%Y-%m-%d %H:%M:%S %z",
    "%Y-%m-%d %H:%M:%S%z",
    "%Y-%m-%d %H:%M:%S,%f",
    "%Y/%m/%d*%H:%M:%S",
    "%Y %b %d %H:%M:%S.%f*%Z",
    "%Y %b %d %H:%M:%S.%f",
    "%Y-%m-%d %H:%M:%S,%f%z",
    "%Y-%m-%d %H:%M:%S.%f",
    "%Y-%m-%d %H:%M:%S.%f%z",
    "%Y-%m-%dT%H:%M:%S.%f",
    "%Y-%m-%dT%H:%M:%S",
    "%Y-%m-%dT%H:%M:%S%z",
    "%Y-%m-%dT%H:%M:%S.%f",
    "%Y-%m-%dT%H:%M:%S",
    "%Y-%m-%d*%H:%M:%S:%f",
    "%Y-%m-%d*%H:%M:%S",
    "%y-%m-%d %H:%M:%S,%f %z",
    "%y-%m-%d %H:%M:%S,%f",
    "%y-%m-%d %H:%M:%S",
    "%y/%m/%d %H:%M:%S",
    "%y%m%d %H:%M:%S",
    "%Y%m%d %H:%M:%S.%f",
    "%m/%d/%y*%H:%M:%S",
    "%m/%d/%Y*%H:%M:%S",
    "%m/%d/%Y*%H:%M:%S*%f",
    "%m/%d/%y %H:%M:%S %z",
    "%m/%d/%Y %H:%M:%S %z",
    "%H:%M:%S",
    "%H:%M:%S.%f",
    "%H:%M:%S,%f",
    "%d/%b %H:%M:%S,%f",
    "%d/%b/%Y:%H:%M:%S",
    "%d/%b/%Y %H:%M:%S",
    "%d-%b-%Y %H:%M:%S",
    "%d-%b-%Y %H:%M:%S.%f",
    "%d %b %Y %H:%M:%S",
    "%d %b %Y %H:%M:%S*%f",
    "%m%d_%H:%M:%S",
    "%m%d_%H:%M:%S.%f",
    "%m/%d/%Y %I:%M:%S %p:%f",
    "%m/%d/%Y %I:%M:%S %p",
]


class BadLookup(Exception):
    """Error returned when a resource lookup fails."""


class PantherBadInput(Exception):
    """Error returned when a Panther helper function is provided bad input."""


def resolve_timestamp_string(timestamp: str) -> Optional[datetime]:
    """Auto Time Resolution"""
    if not timestamp:
        return None

    # Removes weird single-quotes used in some timestamp formats
    ts_format = timestamp.replace("'", "")
    # Attempt to resolve timestamp format
    for each_format in TIME_FORMATS:
        try:
            return datetime.strptime(ts_format, each_format)
        except (ValueError, TypeError):
            continue
    try:
        return parser.parse(timestamp)
    except (ValueError, TypeError, parser.ParserError):
        pass

    # Attempt to resolve epoch format
    # Since datetime.utcfromtimestamp supports 9 through 12 digit epoch timestamps
    # and we only want the first 12 digits.
    match = re.match(EPOCH_REGEX, timestamp)
    if match.group(0) != "":
        try:
            return datetime.utcfromtimestamp(float(match.group(0)))
        except (ValueError, TypeError):
            return None
    return None


def get_s3_arn_by_name(name: str) -> str:
    """This function is used to construct an s3 bucket ARN from its name."""
    if name == "":
        raise PantherBadInput("s3 name cannot be blank")
    return "arn:aws:s3:::" + name


def s3_lookup_by_name(name: str) -> Dict[str, Any]:
    """This function is used to get an S3 bucket resource from just its name."""
    return resource_lookup(get_s3_arn_by_name(name))


def resource_table() -> boto3.resource:
    """Lazily build resource table"""
    # pylint: disable=global-statement
    global _RESOURCE_TABLE
    if not _RESOURCE_TABLE:
        # pylint: disable=no-member
        _RESOURCE_TABLE = boto3.resource(
            "dynamodb",
            endpoint_url="https://dynamodb" + FIPS_SUFFIX if FIPS_ENABLED else None,
        ).Table("panther-resources")
    return _RESOURCE_TABLE


def resource_lookup(resource_id: str) -> Dict[str, Any]:
    """This function is used to get a resource from the resources-api based on its resourceID."""
    # Validate input so we can provide meaningful error messages to users
    if resource_id == "":
        raise PantherBadInput("resourceId cannot be blank")

    # Get the item from dynamo
    response = resource_table().get_item(Key={"id": resource_id})

    # Check if dynamo failed
    status_code = response["ResponseMetadata"]["HTTPStatusCode"]
    if status_code != 200:
        raise BadLookup("dynamodb - " + str(status_code) + " HTTPStatusCode")

    # Check if the item was found
    if "Item" not in response:
        raise BadLookup(resource_id + " not found")

    # Return just the attributes of the item
    return response["Item"]["attributes"]


def ttl_expired(response: dict) -> bool:
    """
    Global `ttl_expired` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import ttl_expired`.
    """
    return caching.ttl_expired(response)


def get_counter(key: str, force_ttl_check: bool = False) -> int:
    """
    Global `get_counter` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import get_counter`.
    """
    return caching.get_counter(key=key, force_ttl_check=force_ttl_check)


def increment_counter(key: str, val: int = 1) -> int:
    """
    Global `increment_counter` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import increment_counter`.
    """
    return caching.increment_counter(key=key, val=val)


def reset_counter(key: str) -> None:
    """
    Global `reset_counter` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import reset_counter`.
    """
    return caching.reset_counter(key=key)


def set_key_expiration(key: str, epoch_seconds: int) -> None:
    """
    Global `set_key_expiration` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import set_key_expiration`.
    """
    return caching.set_key_expiration(key=key, epoch_seconds=epoch_seconds)


def put_dictionary(key: str, val: dict, epoch_seconds: int = None):
    """
    Global `put_dictionary` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import put_dictionary`.
    """
    return caching.put_dictionary(key=key, val=val, epoch_seconds=epoch_seconds)


def get_dictionary(key: str, force_ttl_check: bool = False) -> dict:
    """
    Global `get_dictionary` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import get_dictionary`.
    """
    return caching.get_dictionary(key=key, force_ttl_check=force_ttl_check)


def get_string_set(key: str, force_ttl_check: bool = False) -> Set[str]:
    """
    Global `get_string_set` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import get_string_set`.
    """
    return caching.get_string_set(key=key, force_ttl_check=force_ttl_check)


def put_string_set(key: str, val: Sequence[str], epoch_seconds: int = None) -> None:
    """
    Global `put_string_set` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import put_string_set`.
    """
    return caching.put_string_set(key=key, val=val, epoch_seconds=epoch_seconds)


def add_to_string_set(key: str, val: Union[str, Sequence[str]]) -> Set[str]:
    """
    Global `add_to_string_set` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import add_to_string_set`.
    """
    return caching.add_to_string_set(key=key, val=val)


def remove_from_string_set(key: str, val: Union[str, Sequence[str]]) -> Set[str]:
    """
    Global `remove_from_string_set` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import remove_from_string_set`.
    """
    return caching.remove_from_string_set(key=key, val=val)


def reset_string_set(key: str) -> None:
    """
    Global `reset_string_set` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import reset_string_set`.
    """
    return caching.reset_string_set(key=key)


def evaluate_threshold(key: str, threshold: int = 10, expiry_seconds: int = 3600) -> bool:
    """
    Global `evaluate_threshold` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import evaluate_threshold`.
    """
    return caching.evaluate_threshold(key=key, threshold=threshold, expiry_seconds=expiry_seconds)


def check_account_age(key):
    """
    Global `check_account_age` is DEPRECATED.
    Instead, use `from panther_detection_helpers.caching import check_account_age`.
    """
    return caching.check_account_age(key=key)


def km_between_ipinfo_loc(ipinfo_loc_one: dict, ipinfo_loc_two: dict):
    """
    Compute the number of kilometers between two ipinfo_location enrichments
    This uses a haversine computation which is imperfect and holds the benefit
    of being supportable via stdlib. At polar opposites, haversine might be
    0.3-0.5% off
    See also https://en.wikipedia.org/wiki/Haversine_formula
    See also https://stackoverflow.com/a/19412565
    See also https://www.sunearthtools.com/tools/distance.php
    """
    if not set({"lat", "lng"}).issubset(set(ipinfo_loc_one.keys())):
        # input ipinfo_loc_one doesn't have lat and lng keys
        return None
    if not set({"lat", "lng"}).issubset(set(ipinfo_loc_two.keys())):
        # input ipinfo_loc_two doesn't have lat and lng keys
        return None
    lat_1 = radians(float(ipinfo_loc_one.get("lat")))
    lng_1 = radians(float(ipinfo_loc_one.get("lng")))
    lat_2 = radians(float(ipinfo_loc_two.get("lat")))
    lng_2 = radians(float(ipinfo_loc_two.get("lng")))
    # radius of the earth in kms
    radius = 6372.795477598
    lng_diff = lng_2 - lng_1
    lat_diff = lat_2 - lat_1

    step_1 = sin(lat_diff / 2) ** 2 + cos(lat_1) * cos(lat_2) * sin(lng_diff / 2) ** 2
    step_2 = 2 * atan2(sqrt(step_1), sqrt(1 - step_1))
    distance = radius * step_2
    return distance


def geoinfo_from_ip(ip: str) -> dict:  # pylint: disable=invalid-name
    """
    Looks up the geolocation of an IP address using ipinfo.io

    Example ipinfo output:
    {
      "ip": "1.1.1.1",
      "hostname": "one.one.one.one",
      "anycast": true,
      "city": "Miami",
      "region": "Florida",
      "country": "US",
      "loc": "25.7867,-80.1800",
      "org": "AS13335 Cloudflare, Inc.",
      "postal": "33132",
      "timezone": "America/New_York",
      "readme": "https://ipinfo.io/missingauth"
    }
    """
    valid_ip = ip_address(ip)
    url = f"https://ipinfo.io/{valid_ip}/json"
    resp = requests.get(url, timeout=5)
    if resp.status_code != 200:
        # pylint: disable=broad-exception-raised
        raise Exception(f"Geo lookup failed: GET {url} returned {resp.status_code}")
    geoinfo = json.loads(resp.text)
    return geoinfo


def geoinfo_from_ip_formatted(ip: str) -> str:  # pylint: disable=invalid-name
    """Formatting wrapper for geoinfo_from_ip for use in human-readable text"""
    geoinfo = geoinfo_from_ip(ip)
    return f"{geoinfo.get('ip')} in {geoinfo.get('city')}, {geoinfo.get('region')} in {geoinfo.get('country')}"


# returns the difference between time1 and later time 2 in human-readable time period string
def time_delta(time1, time2: str) -> str:
    time1_truncated = nano_to_micro(time1)
    time2_truncated = nano_to_micro(time2)
    delta_timedelta = resolve_timestamp_string(time2_truncated) - resolve_timestamp_string(time1_truncated)
    days = delta_timedelta.days
    hours, remainder = divmod(delta_timedelta.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    delta = ""
    if days > 0:
        delta = f"{days} day(s) "
    if hours > 0:
        delta = "".join([delta, f"{hours} hour(s) "])
    if minutes > 0:
        delta = "".join([delta, f"{minutes} minute(s) "])
    if seconds > 0:
        delta = "".join([delta, f"{seconds} second(s)"])
    return delta


def nano_to_micro(time_str: str) -> str:
    parts = time_str.split(":")
    # pylint: disable=consider-using-f-string
    parts[-1] = f"{float(parts[-1]):06f}"
    return ":".join(parts)


# adds parsing delay to an alert_context
def add_parse_delay(event, context: dict) -> dict:
    parsing_delay = time_delta(event.get("p_event_time"), event.get("p_parse_time"))
    context["parseDelay"] = f"{parsing_delay}"
    return context


# When a single item is loaded from json, it is loaded as a single item
# When a list of items is loaded from json, it is loaded as a list of that item
# When we want to iterate over something that could be a single item or a list
# of items we can use listify and just continue as if it's always a list
def listify(maybe_list):
    try:
        iter(maybe_list)
    except TypeError:
        # not a list
        return [maybe_list]
    # either a list or string
    return [maybe_list] if isinstance(maybe_list, (str, bytes, dict)) else maybe_list



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardAWSVPCDns(DataModel):
    id: str = "Standard.AWS.VPCDns"
    display_name: str = "AWS VPC DNS"
    enabled: bool = True
    log_types: list[str] = [LogType.AWS_VPC_DNS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="source_ip", path="srcAddr"),
        DataModelMapping(name="source_port", path="srcPort"),
        DataModelMapping(name="dns_query", path="query_name"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

gcp_workload_identity_pool_createdor_updated_tests: list[RuleTest] = [
    RuleTest(
        name="DeleteWorkloadIdentityPoolProvider-False",
        expected_result=False,
        log={
            "insertId": "1h09dxwe33il5",
            "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "projects/1234567890123/locations/global/workloadIdentityPools/test-pool/operations/bigarrpp32vamefyvthk4ay000000000",
                "producer": "iam.googleapis.com",
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "user@example.com",
                    "principalSubject": "user:user@example.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.workloadIdentityPools.delete",
                        "resource": "projects/test-project/locations/global/workloadIdentityPools/test-pool",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.v1.WorkloadIdentityPools.DeleteWorkloadIdentityPool",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.DeleteWorkloadIdentityPoolRequest",
                    "name": "projects/test-project/locations/global/workloadIdentityPools/test-pool",
                },
                "requestMetadata": {
                    "callerIp": "07da:0994:97fb:8db1:c68f:c109:fcdd:d594",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0,gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-11-17T18:58:13.519015485Z"},
                },
                "resourceName": "projects/test-project/locations/global/workloadIdentityPools/test-pool",
                "serviceName": "iam.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T18:58:14.565078208Z",
            "resource": {
                "labels": {
                    "method": "google.iam.v1.WorkloadIdentityPools.DeleteWorkloadIdentityPool",
                    "project_id": "test-project",
                    "service": "iam.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T18:58:13.511621185Z",
        },
    ),
    RuleTest(
        name="UpdateWorkloadIdentityPoolProvider-True",
        expected_result=True,
        log={
            "insertId": "1plwiv7e2lak8",
            "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "projects/1234567890123/locations/global/workloadIdentityPools/test-pool/providers/test-project/operations/bifqr6xo32vameeqtose200000000000",
                "producer": "iam.googleapis.com",
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "user@example.com",
                    "principalSubject": "user:user@example.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.workloadIdentityPoolProviders.update",
                        "resource": "projects/test-project/locations/global/workloadIdentityPools/test-pool/providers/test-project",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.v1.WorkloadIdentityPools.UpdateWorkloadIdentityPoolProvider",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.UpdateWorkloadIdentityPoolProviderRequest",
                    "updateMask": "displayName,disabled,attributeMapping,attributeCondition,aws.accountId",
                    "workloadIdentityPoolProvider": {
                        "attributeCondition": "'admins' in google.groups",
                        "attributeMapping": {
                            "attribute.aws_role": "assertion.arn.contains('assumed-role') ? assertion.arn.extract('{account_arn}assumed-role/') + 'assumed-role/' + assertion.arn.extract('assumed-role/{role_name}/') : assertion.arn",
                            "google.subject": "assertion.arn",
                        },
                        "aws": {"accountId": "123456789012"},
                        "disabled": False,
                        "displayName": "Test Provider",
                        "name": "projects/test-project/locations/global/workloadIdentityPools/test-pool/providers/test-project",
                    },
                },
                "requestMetadata": {
                    "callerIp": "07da:0994:97fb:8db1:c68f:c109:fcdd:d594",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0,gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-11-17T18:56:57.745203848Z"},
                },
                "resourceName": "projects/test-project/locations/global/workloadIdentityPools/test-pool/providers/test-project",
                "serviceName": "iam.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T18:56:58.871491875Z",
            "resource": {
                "labels": {
                    "method": "google.iam.v1.WorkloadIdentityPools.UpdateWorkloadIdentityPoolProvider",
                    "project_id": "test-project",
                    "service": "iam.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T18:56:57.730630771Z",
        },
    ),
    RuleTest(
        name="CreateWorkloadIdentityPoolProvider-True",
        expected_result=True,
        log={
            "insertId": "11gmdk5e1ne4r",
            "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "projects/1234567890123/locations/global/workloadIdentityPools/test-pool/providers/test-project/operations/bigarpxj32vamehaqcf5oai000000000",
                "producer": "iam.googleapis.com",
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "user@example.com",
                    "principalSubject": "user:user@example.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.workloadIdentityPoolProviders.create",
                        "resource": "projects/test-project/locations/global/workloadIdentityPools/test-pool",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.v1.WorkloadIdentityPools.CreateWorkloadIdentityPoolProvider",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.CreateWorkloadIdentityPoolProviderRequest",
                    "parent": "projects/test-project/locations/global/workloadIdentityPools/test-pool",
                    "workloadIdentityPoolProvider": {
                        "attributeCondition": "",
                        "attributeMapping": {
                            "attribute.aws_role": "assertion.arn.contains('assumed-role') ? assertion.arn.extract('{account_arn}assumed-role/') + 'assumed-role/' + assertion.arn.extract('assumed-role/{role_name}/') : assertion.arn",
                            "google.subject": "assertion.arn",
                        },
                        "aws": {"accountId": "123456789012"},
                        "disabled": False,
                        "displayName": "Test Provider",
                    },
                    "workloadIdentityPoolProviderId": "test-project",
                },
                "requestMetadata": {
                    "callerIp": "07da:0994:97fb:8db1:c68f:c109:fcdd:d594",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0,gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-11-17T18:45:17.961743198Z"},
                },
                "resourceLocation": {"currentLocations": ["global"]},
                "resourceName": "projects/test-project/locations/global/workloadIdentityPools/test-pool",
                "serviceName": "iam.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T18:45:19.404664001Z",
            "resource": {
                "labels": {
                    "method": "google.iam.v1.WorkloadIdentityPools.CreateWorkloadIdentityPoolProvider",
                    "project_id": "test-project",
                    "service": "iam.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T18:45:17.952414168Z",
        },
    ),
]


@panther_managed
class GCPWorkloadIdentityPoolCreatedorUpdated(Rule):
    id = "GCP.Workload.Identity.Pool.Created.or.Updated-prototype"
    display_name = "GCP Workload Identity Pool Created or Updated"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["Account Manipulation", "Additional Cloud Roles", "GCP", "Privilege Escalation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136.003", "TA0003:T1098.003", "TA0004:T1098.003"]}
    default_severity = Severity.HIGH
    default_runbook = "Ensure that the Workload Identity Pool creation or modification was expected. Adversaries may use this to persist or allow additional access or escalate their privilege.\n"
    default_reference = (
        "https://medium.com/google-cloud/detection-of-inbound-sso-persistence-techniques-in-gcp-c56f7b2a588b"
    )
    tests = gcp_workload_identity_pool_createdor_updated_tests
    METHODS = [
        "google.iam.v1.WorkloadIdentityPools.CreateWorkloadIdentityPoolProvider",
        "google.iam.v1.WorkloadIdentityPools.UpdateWorkloadIdentityPoolProvider",
    ]

    def rule(self, event):
        return event.deep_get("protoPayload", "methodName", default="") in self.METHODS

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>").split("/")
        workload_identity_pool = resource[resource.index("workloadIdentityPools") + 1]
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"GCP: [{actor}] created or updated workforce pool [{workload_identity_pool}] in project [{project_id}]"

    def alert_context(self, event):
        return event.deep_get("protoPayload", "request", "workloadIdentityPoolProvider", default={})



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_device_compromise_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Mobile Event",
        expected_result=False,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_REGISTER_UNREGISTER_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
    RuleTest(
        name="Suspicious Activity Shows not Compromised",
        expected_result=False,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_COMPROMISED_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "DEVICE_COMPROMISED_STATE": "NOT_COMPROMISED"},
        },
    ),
    RuleTest(
        name="Suspicious Activity Shows Compromised",
        expected_result=True,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_COMPROMISED_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "DEVICE_COMPROMISED_STATE": "COMPROMISED"},
        },
    ),
]


@panther_managed
class GSuiteDeviceCompromise(Rule):
    id = "GSuite.DeviceCompromise-prototype"
    display_name = "GSuite User Device Compromised"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.MEDIUM
    default_description = "GSuite reported a user's device has been compromised.\n"
    default_reference = "https://support.google.com/a/answer/7562165?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Have the user change their passwords and reset the device.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_device_compromise_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "mobile":
            return False
        if event.get("name") == "DEVICE_COMPROMISED_EVENT":
            return bool(deep_get(event, "parameters", "DEVICE_COMPROMISED_STATE") == "COMPROMISED")
        return False

    def title(self, event):
        return (
            f"User [{deep_get(event, 'parameters', 'USER_EMAIL', default='<UNKNOWN_USER>')}]'s device was compromised"
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import lookup_aws_account_name

aws_console_root_login_failed_tests: list[RuleTest] = [
    RuleTest(
        name="Failed Root Login",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "Root",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:root",
                "accountId": "123456789012",
                "userName": "root",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Failure"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Successful Login",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "Root",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:root",
                "accountId": "123456789012",
                "userName": "root",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "No",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Non-Login Event",
        expected_result=False,
        log={
            "eventVersion": "1.06",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:user/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "dynamodb.amazonaws.com",
            "eventName": "DescribeTable",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"tableName": "table"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": True,
            "resources": [{"accountId": "123456789012", "type": "AWS::DynamoDB::Table", "ARN": "arn::::table/table"}],
            "eventType": "AwsApiCall",
            "apiVersion": "2012-08-10",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSConsoleRootLoginFailed(Rule):
    id = "AWS.Console.RootLoginFailed-prototype"
    display_name = "Failed Root Console Login"
    dedup_period_minutes = 15
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = [
        "AWS",
        "Identity & Access Management",
        "Authentication",
        "DemoThreatHunting",
        "Credential Access:Brute Force",
    ]
    threshold = 5
    reports = {"CIS": ["3.6"], "MITRE ATT&CK": ["TA0006:T1110"]}
    default_severity = Severity.HIGH
    default_description = "A Root console login failed."
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-console-login-failed"
    default_reference = "https://amzn.to/3aMSmTd"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_console_root_login_failed_tests

    def rule(self, event):
        return (
            event.get("eventName") == "ConsoleLogin"
            and deep_get(event, "userIdentity", "type") == "Root"
            and (deep_get(event, "responseElements", "ConsoleLogin") == "Failure")
        )

    def title(self, event):
        return f"AWS root login failed from [{event.get('sourceIPAddress')}] in account [{lookup_aws_account_name(event.get('recipientAccountId'))}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcpk8_s_privileged_pod_created_tests: list[RuleTest] = [
    RuleTest(
        name="Privileged Pod Created",
        expected_result=True,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {},
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test-privileged-pod",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {"name": "test-privileged-pod", "namespace": "default"},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "nginx",
                                "resources": {},
                                "securityContext": {"privileged": True},
                            },
                        ],
                        "securityContext": {},
                    },
                    "status": {},
                },
                "requestMetadata": {"callerIP": "1.2.3.4"},
                "resourceName": "core/v1/namespaces/default/pods/test-privileged-pod",
                "response": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "nginx",
                                "resources": {},
                                "securityContext": {"privileged": True},
                            },
                        ],
                        "securityContext": {},
                        "serviceAccount": "default",
                        "serviceAccountName": "default",
                        "terminationGracePeriodSeconds": 30,
                    },
                    "status": {},
                },
                "serviceName": "k8s.io",
                "status": {},
            },
            "receiveTimestamp": "2024-02-13 12:45:20.058795785",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-13 12:45:06.073905000",
        },
    ),
    RuleTest(
        name="Run-As-Root Pod Created",
        expected_result=True,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {},
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test-runasroot-pod",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "nginx",
                                "resources": {},
                                "securityContext": {"runAsNonRoot": False},
                            },
                        ],
                    },
                    "status": {},
                },
                "requestMetadata": {"callerIP": "1.2.3.4"},
                "resourceName": "core/v1/namespaces/default/pods/test-runasroot-pod",
                "response": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "nginx",
                                "resources": {},
                                "securityContext": {"runAsNonRoot": False},
                            },
                        ],
                    },
                    "status": {"phase": "Pending", "qosClass": "BestEffort"},
                },
                "serviceName": "k8s.io",
                "status": {},
            },
            "receiveTimestamp": "2024-02-13 13:13:53.113465457",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-13 13:13:45.363388000",
        },
    ),
    RuleTest(
        name="Non-Privileged Pod Created",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "7f8c5bec-01ff-4079-97e3-065ac34e10e8",
                "last": True,
                "producer": "k8s.io",
            },
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test-non-privileged-pod",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {"name": "test-non-privileged-pod", "namespace": "default"},
                    "spec": {
                        "containers": [
                            {"image": "nginx", "imagePullPolicy": "Always", "name": "nginx", "resources": {}},
                        ],
                    },
                    "status": {},
                },
                "requestMetadata": {"callerIP": "1.2.3.4"},
                "resourceName": "core/v1/namespaces/default/pods/test-non-privileged-pod",
                "response": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {},
                    "spec": {
                        "containers": [
                            {"image": "nginx", "imagePullPolicy": "Always", "name": "nginx", "resources": {}},
                        ],
                    },
                    "status": {},
                },
                "serviceName": "k8s.io",
                "status": {},
            },
            "receiveTimestamp": "2024-02-13 13:07:54.642331675",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-13 13:07:29.505948000",
        },
    ),
    RuleTest(
        name="Error Creating Pod",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "john.doe@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test-privileged-pod",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {"name": "test-privileged-pod", "namespace": "default"},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "nginx",
                                "resources": {},
                                "securityContext": {"runAsNonRoot": False},
                            },
                        ],
                    },
                    "status": {},
                },
                "requestMetadata": {"callerIP": "1.2.3.4"},
                "resourceName": "core/v1/namespaces/default/pods/test-privileged-pod",
                "response": {
                    "@type": "core.k8s.io/v1.Status",
                    "apiVersion": "v1",
                    "code": 409,
                    "details": {"kind": "pods", "name": "test-privileged-pod"},
                    "kind": "Status",
                    "message": 'pods "test-privileged-pod" already exists',
                    "metadata": {},
                    "reason": "AlreadyExists",
                    "status": "Failure",
                },
                "serviceName": "k8s.io",
                "status": {"code": 10, "message": 'pods "test-privileged-pod" already exists'},
            },
            "receiveTimestamp": "2024-02-13 13:13:33.486605432",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-13 13:13:24.079140000",
        },
    ),
]


@panther_managed
class GCPK8SPrivilegedPodCreated(Rule):
    id = "GCP.K8S.Privileged.Pod.Created-prototype"
    display_name = "GCP K8S Privileged Pod Created"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_description = "Alerts when a user creates privileged pod. These particular pods have full access to the host’s namespace and devices, have the ability to exploit the kernel, have dangerous linux capabilities, and can be a powerful launching point for further attacks. In the event of a successful container escape where a user is operating with root privileges, the attacker retains this role on the node.\n"
    default_runbook = "Investigate the reason of creating privileged pod. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://www.golinuxcloud.com/kubernetes-privileged-pod-examples/"
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    tests = gcpk8_s_privileged_pod_created_tests

    def rule(self, event):
        if deep_get(event, "protoPayload", "response", "status") == "Failure":
            return False
        if deep_get(event, "protoPayload", "methodName") != "io.k8s.core.v1.pods.create":
            return False
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        containers_info = deep_walk(event, "protoPayload", "response", "spec", "containers")
        for auth in authorization_info:
            if auth.get("permission") == "io.k8s.core.v1.pods.create" and auth.get("granted") is True:
                for security_context in containers_info:
                    if (
                        deep_get(security_context, "securityContext", "privileged") is True
                        or deep_get(security_context, "securityContext", "runAsNonRoot") is False
                    ):
                        return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        pod_name = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created a privileged pod [{pod_name}] in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        containers_info = deep_walk(event, "protoPayload", "response", "spec", "containers", default=[])
        context["pod_security_context"] = [i.get("securityContext") for i in containers_info]
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_many_pages_exported_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-06-02T20:16:41.217Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "mac-desktop",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            },
        },
    ),
    RuleTest(
        name="Many Pages Exported",
        expected_result=True,
        log={
            "event": {
                "actor": {
                    "id": "bd37477c-869d-418b-abdb-0fc727b38b5e",
                    "object": "user",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                    "type": "person",
                },
                "details": {
                    "parent": {"type": "workspace_id", "workspace_id": "ab99as87-6abc-4dcf-808b-111999882299"},
                    "target": {"page_id": "3cd2c560-d1b9-474e-b46e-gh8899002763", "type": "page_id"},
                },
                "id": "d4b9963f-12a8-4b01-b597-233a140abf5e",
                "ip_address": "12.12.12.12",
                "platform": "web",
                "timestamp": "2023-06-01 18:57:07.486000000",
                "type": "page.exported",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            },
        },
    ),
]


@panther_managed
class NotionManyPagesExported(Rule):
    id = "Notion.Many.Pages.Exported-prototype"
    display_name = "Notion Many Pages Exported"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Security", "Data Exfiltration"]
    default_severity = Severity.HIGH
    default_description = "A Notion User exported multiple pages."
    threshold = 10
    default_runbook = "Possible Data Exfiltration. Follow up with the Notion User to determine if this was done for a valid business reason."
    default_reference = "https://www.notion.so/help/export-your-content"
    tests = notion_many_pages_exported_tests

    def rule(self, event):
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") == "page.exported"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        return f"Notion User [{user}] exported multiple pages."

    def alert_context(self, event):
        context = notion_alert_context(event)
        page_id = event.deep_get("event", "details", "target", "page_id", default="<NO_PAGE_ID_FOUND>")
        context["page_id"] = page_id
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

awsiam_access_key_compromised_tests: list[RuleTest] = [
    RuleTest(
        name="An AWS Access Key was Uploaded to Github",
        expected_result=True,
        log={
            "eventSource": "iam.amazonaws.com",
            "recipientAccountId": "123456789012",
            "responseElements": None,
            "userIdentity": {
                "type": "IAMUser",
                "userName": "compromised_user",
                "principalId": "XXXXXXXXXXXXXXXXXXX",
                "accessKeyId": "XXXXXXXXXXXXXXXXXXXXX",
                "arn": "arn:aws:iam::123456789012:user/compromised_user",
                "accountId": "123456789012",
            },
            "eventName": "PutUserPolicy",
            "eventVersion": "1.05",
            "userAgent": "aws-internal/3 aws-sdk-java/1.11.706 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
            "requestParameters": {
                "policyDocument": '{"Version":"2012-10-17","Statement":[{"Sid":"Stmt1538161409","Effect":"Deny","Action":["lambda:CreateFunction","iam:AttachUserPolicy","iam:PutUserPolicy","organizations:InviteAccountToOrganization","ec2:RunInstances","iam:DetachUserPolicy","iam:CreateUser","lightsail:Create*","lightsail:Update*","ec2:StartInstances","ec2:RequestSpotInstances","iam:ChangePassword","iam:CreateLoginProfile","organizations:CreateOrganization","organizations:CreateAccount","lightsail:Delete*","iam:AttachGroupPolicy","iam:CreateAccessKey","iam:UpdateUser","iam:UpdateAccountPasswordPolicy","iam:DeleteUserPolicy","iam:PutUserPermissionsBoundary","iam:UpdateAccessKey","lightsail:DownloadDefaultKeyPair","iam:CreateInstanceProfile","lightsail:Start*","lightsail:GetInstanceAccessDetails","iam:CreateRole","iam:PutGroupPolicy","iam:AttachRolePolicy"],"Resource":["*"]}]}',
                "userName": "compromised_user",
                "policyName": "AWSExposedCredentialPolicy_DO_NOT_REMOVE",
            },
            "eventID": "1c2a53d1-58cc-41b3-85b8-bd7565370e0d",
            "eventType": "AwsApiCall",
            "sourceIPAddress": "72.21.217.97",
            "awsRegion": "us-east-1",
            "requestID": "27ca92a5-61cc-44aa-b875-042a25310064",
            "eventTime": "2020-04-10T06:22:08Z",
        },
    ),
    RuleTest(
        name="Request Param is null",
        expected_result=False,
        log={
            "eventSource": "iam.amazonaws.com",
            "recipientAccountId": "123456789012",
            "responseElements": None,
            "userIdentity": {
                "type": "IAMUser",
                "userName": "compromised_user",
                "principalId": "XXXXXXXXXXXXXXXXXXX",
                "accessKeyId": "XXXXXXXXXXXXXXXXXXXXX",
                "arn": "arn:aws:iam::123456789012:user/compromised_user",
                "accountId": "123456789012",
            },
            "eventName": "PutUserPolicy",
            "eventVersion": "1.05",
            "userAgent": "aws-internal/3 aws-sdk-java/1.11.706 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
            "requestParameters": None,
            "eventID": "1c2a53d1-58cc-41b3-85b8-bd7565370e0d",
            "eventType": "AwsApiCall",
            "sourceIPAddress": "72.21.217.97",
            "awsRegion": "us-east-1",
            "requestID": "27ca92a5-61cc-44aa-b875-042a25310064",
            "eventTime": "2020-04-10T06:22:08Z",
        },
    ),
]


@panther_managed
class AWSIAMAccessKeyCompromised(Rule):
    id = "AWS.IAM.AccessKeyCompromised-prototype"
    display_name = "AWS Access Key Uploaded to Github"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    tags = ["AWS", "Credential Access:Unsecured Credentials"]
    default_severity = Severity.HIGH
    default_description = "A users static AWS API key was uploaded to a public github repo."
    default_runbook = "Determine the key owner, disable/delete key, and delete the user to resolve the AWS case. If user needs a new IAM give them a stern talking to first."
    default_reference = "https://docs.github.com/en/code-security/secret-scanning/about-secret-scanning"
    tests = awsiam_access_key_compromised_tests
    EXPOSED_CRED_POLICY = "AWSExposedCredentialPolicy_DO_NOT_REMOVE"

    def rule(self, event):
        request_params = event.get("requestParameters", {})
        if request_params:
            return (
                event.get("eventName") == "PutUserPolicy"
                and request_params.get("policyName") == self.EXPOSED_CRED_POLICY
            )
        return False

    def dedup(self, event):
        return deep_get(event, "userIdentity", "userName")

    def title(self, event):
        return f"{self.dedup(event)}'s access key ID [{deep_get(event, 'userIdentity', 'accessKeyId')}] was uploaded to a public GitHub repo"

    def alert_context(self, event):
        return aws_rule_context(event)



"""Used to define global helpers and variables for Duo events."""

import json
from json import JSONDecodeError


def deserialize_administrator_log_event_description(event: dict) -> dict:
    """
    Intelligently try and decode a field that is usually stringified json into a python dict.

    This description field seems to take the form of stringified json, So this function
    makes an educated guess on how to transform it into a useful dict structure. and is resilient
    if it's not formed that way
    """
    desc_string = event.get("description", "")
    if desc_string.startswith("{"):
        try:
            # This should be the happy path if the duo docs are correct
            return json.loads(desc_string)
        except JSONDecodeError:
            pass
    elif desc_string.startswith("["):
        try:
            return {"items": json.loads(desc_string)}
        except JSONDecodeError:
            pass

    return {"value": desc_string}


def duo_alert_context(event):
    return {
        "action": event.get("action", "<action_not_found>"),
        "description": event.get("description", "<description_not_found>"),
        "username": event.get("username", "<username_not_found>"),
        "timestamp": event.get("timestamp", "<timestamp_not_found>"),
    }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

awsec2_startup_script_change_tests: list[RuleTest] = [
    RuleTest(
        name="ModifyInstanceAttribute-NoUserData",
        expected_result=False,
        log={
            "awsregion": "us-east-1",
            "eventid": "abc-123",
            "eventname": "ModifyInstanceAttribute",
            "eventsource": "ec2.amazonaws.com",
            "eventtime": "2022-07-17 04:50:23",
            "eventtype": "AwsApiCall",
            "eventversion": "1.08",
            "p_any_aws_instance_ids": ["testinstanceid"],
            "p_event_time": "2022-07-17 04:50:23",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-07-17 04:55:11.788",
            "requestParameters": {"instanceId": "testinstanceid", "instanceType": {"value": "t3.nano"}},
        },
    ),
    RuleTest(
        name="ModifyInstanceAttributeUserdata",
        expected_result=True,
        log={
            "awsRegion": "us-east-2",
            "eventCategory": "Management",
            "eventName": "ModifyInstanceAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-09-30 15:11:25.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "0123456789",
            "requestID": "example-id",
            "requestParameters": {"instanceId": "i-012345abcde", "userData": "<sensitiveDataRemoved>"},
            "responseElements": {"_return": True, "requestId": "012345abcdef"},
            "sourceIPAddress": "1.2.3.4",
            "tlsDetails": {
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "ec2.us-east-2.amazonaws.com",
                "tlsVersion": "TLSv1.2",
            },
            "userAgent": "aws sdk",
            "userIdentity": {
                "accessKeyId": "ABCDEXAMPLE123",
                "accountId": "0123456789",
                "arn": "arn:aws:sts::0123456789:assumed-role/Role-us-1/CodeBuild",
                "principalId": "ABCDEXAMPLE:CodeBuild",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-30T14:52:26Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "0123456789",
                        "arn": "arn:aws:iam::0123456789:role/CodeBuild-US-East",
                        "principalId": "AROAQUW22FGSKBTQ7R5HP",
                        "type": "Role",
                        "userName": "CodeBuild-US-East",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="NoModifyInstanceAttribute",
        expected_result=False,
        log={
            "awsregion": "us-east-1",
            "eventid": "abc-123",
            "eventname": "ModifyImageAttribute",
            "eventsource": "ec2.amazonaws.com",
            "eventtime": "2022-07-17 04:50:23",
            "eventtype": "AwsApiCall",
            "eventversion": "1.08",
            "p_any_aws_instance_ids": ["testinstanceid"],
            "p_event_time": "2022-07-17 04:50:23",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-07-17 04:55:11.788",
            "requestParameters": {"instanceId": "testinstanceid", "instanceType": {"value": "t3.nano"}},
        },
    ),
]


@panther_managed
class AWSEC2StartupScriptChange(Rule):
    default_description = "Detects changes to the EC2 instance startup script. The shell script will be executed as root/SYSTEM every time the specific instances are booted up."
    display_name = "AWS EC2 Startup Script Change"
    reports = {"MITRE ATT&CK": ["TA0002:T1059"]}
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html#user-data-shell-scripts"
    default_severity = Severity.HIGH
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.Startup.Script.Change-prototype"
    tests = awsec2_startup_script_change_tests

    def rule(self, event):
        if event.get("eventName") == "ModifyInstanceAttribute" and deep_get(event, "requestParameters", "userData"):
            return True
        return False

    def title(self, event):
        return f"[{deep_get(event, 'userIdentity', 'arn')}] modified the startup script for  [{deep_get(event, 'requestParameters', 'instanceId')}] in [{event.get('recipientAccountId')}] - [{event.get('awsRegion')}]"

    def dedup(self, event):
        return deep_get(event, "requestParameters", "instanceId")

    def alert_context(self, event):
        context = aws_rule_context(event)
        context["instance_ids"] = [deep_get(event, "requestParameters", "instanceId"), "no_instance_id"]
        return context



import datetime

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, pattern_match, pattern_match_list

g_suite_drive_external_file_share_tests: list[RuleTest] = [
    RuleTest(
        name="Dangerous Share of Known Document with a Missing User",
        expected_result=True,
        log={
            "kind": "admin#reports#activity",
            "id": {
                "time": "2020-09-07T15:50:49.617Z",
                "uniqueQualifier": "1111111111111111111",
                "applicationName": "drive",
                "customerId": "C010qxghg",
            },
            "actor": {"email": "example@acme.com", "profileId": "1111111111111111111"},
            "events": [
                {
                    "type": "acl_change",
                    "name": "change_user_access",
                    "parameters": [
                        {"name": "primary_event", "boolValue": True},
                        {"name": "visibility_change", "value": "external"},
                        {"name": "target_user", "value": "outside@acme.com"},
                        {"name": "old_visibility", "value": "private"},
                        {"name": "doc_id", "value": "1111111111111111111"},
                        {"name": "doc_type", "value": "document"},
                        {"name": "doc_title", "value": "Document Title Primary"},
                        {"name": "visibility", "value": "shared_externally"},
                        {"name": "originating_app_id", "value": "1111111111111111111"},
                        {"name": "owner_is_shared_drive", "boolValue": False},
                        {"name": "owner_is_team_drive", "boolValue": False},
                        {"name": "old_value", "multiValue": ["none"]},
                        {"name": "new_value", "multiValue": ["can_edit"]},
                    ],
                },
            ],
        },
    ),
    RuleTest(
        name="Dangerous Share of Unknown Document",
        expected_result=True,
        log={
            "kind": "admin#reports#activity",
            "id": {
                "time": "2020-09-07T15:50:49.617Z",
                "uniqueQualifier": "1111111111111111111",
                "applicationName": "drive",
                "customerId": "C010qxghg",
            },
            "actor": {"email": "example@acme.com", "profileId": "1111111111111111111"},
            "events": [
                {
                    "type": "acl_change",
                    "name": "change_user_access",
                    "parameters": [
                        {"name": "primary_event", "boolValue": True},
                        {"name": "visibility_change", "value": "external"},
                        {"name": "target_user", "value": "alice@external.com"},
                        {"name": "old_visibility", "value": "private"},
                        {"name": "doc_id", "value": "1111111111111111111"},
                        {"name": "doc_type", "value": "document"},
                        {"name": "doc_title", "value": "Untitled document"},
                        {"name": "visibility", "value": "shared_externally"},
                        {"name": "originating_app_id", "value": "1111111111111111111"},
                        {"name": "owner_is_shared_drive", "boolValue": False},
                        {"name": "owner_is_team_drive", "boolValue": False},
                        {"name": "old_value", "multiValue": ["none"]},
                        {"name": "new_value", "multiValue": ["can_edit"]},
                    ],
                },
            ],
        },
    ),
    RuleTest(
        name="Share Allowed by Exception",
        expected_result=False,
        log={
            "kind": "admin#reports#activity",
            "id": {
                "time": "2020-07-07T15:50:49.617Z",
                "uniqueQualifier": "1111111111111111111",
                "applicationName": "drive",
                "customerId": "C010qxghg",
            },
            "actor": {"email": "alice@acme.com", "profileId": "1111111111111111111"},
            "events": [
                {
                    "type": "acl_change",
                    "name": "change_user_access",
                    "parameters": [
                        {"name": "primary_event", "boolValue": True},
                        {"name": "billable", "boolValue": True},
                        {"name": "visibility_change", "value": "external"},
                        {"name": "target_domain", "value": "acme.com"},
                        {"name": "old_visibility", "value": "private"},
                        {"name": "doc_id", "value": "1111111111111111111"},
                        {"name": "doc_type", "value": "document"},
                        {"name": "doc_title", "value": "Document Title Pattern"},
                        {"name": "visibility", "value": "shared_externally"},
                        {"name": "originating_app_id", "value": "1111111111111111111"},
                        {"name": "owner_is_shared_drive", "boolValue": False},
                        {"name": "owner_is_team_drive", "boolValue": False},
                        {"name": "old_value", "multiValue": ["none"]},
                        {"name": "new_value", "multiValue": ["people_within_domain_with_link"]},
                    ],
                },
            ],
        },
    ),
]


@panther_managed
class GSuiteDriveExternalFileShare(Rule):
    id = "GSuite.Drive.ExternalFileShare-prototype"
    display_name = "External GSuite File Share"
    enabled = False
    log_types = [LogType.GSUITE_REPORTS]
    tags = ["GSuite", "Security Control", "Configuration Required", "Collection:Data from Information Repositories"]
    reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    default_severity = Severity.HIGH
    default_description = "An employee shared a sensitive file externally with another organization"
    default_runbook = "Contact the employee who made the share and make sure they redact the access. If the share was legitimate, add to the EXCEPTION_PATTERNS in the detection.\n"
    default_reference = (
        "https://support.google.com/docs/answer/2494822?hl=en&co=GENIE.Platform%3DiOS&sjid=864417124752637253-EU"
    )
    tests = g_suite_drive_external_file_share_tests
    COMPANY_DOMAIN = "your-company-name.com"
    # The glob pattern for the document title (lowercased)
    # All actors allowed to receive the file share
    # Allow any user
    # "all"
    # Allow any user in a specific domain
    # "*@acme.com"
    # The time limit for how long the file share stays valid
    EXCEPTION_PATTERNS = {
        "document title p*": {
            "allowed_for": {"alice@acme.com", "samuel@acme.com", "nathan@acme.com", "barry@acme.com"},
            "allowed_until": datetime.datetime(year=2030, month=6, day=2),
        },
    }

    def _check_acl_change_event(self, actor_email, acl_change_event):
        parameters = {p.get("name", ""): p.get("value") or p.get("multiValue") for p in acl_change_event["parameters"]}
        doc_title = parameters.get("doc_title", "TITLE_UNKNOWN")
        old_visibility = parameters.get("old_visibility", "OLD_VISIBILITY_UNKNOWN")
        new_visibility = parameters.get("visibility", "NEW_VISIBILITY_UNKNOWN")
        target_user = parameters.get("target_user", "USER_UNKNOWN")
        current_time = datetime.datetime.now()
        if (
            new_visibility == "shared_externally"
            and old_visibility == "private"
            and (not target_user.endswith(f"@{self.COMPANY_DOMAIN}"))
        ):
            # This is a dangerous share, check exceptions:
            for pattern, details in self.EXCEPTION_PATTERNS.items():
                doc_title_match = pattern_match(doc_title.lower(), pattern)
                allowed_for_match = pattern_match_list(actor_email, details.get("allowed_for"))
                allowed_for_all_match = details.get("allowed_for") == {"all"}
                if (
                    doc_title_match
                    and (allowed_for_match or allowed_for_all_match)
                    and (current_time < details.get("allowed_until"))
                ):
                    return False
                # No exceptions match.
                # Return the event summary (which is True) to alert & use in title.
                return {"actor": actor_email, "doc_title": doc_title, "target_user": target_user}
        return False

    def rule(self, event):
        application_name = deep_get(event, "id", "applicationName")
        events = event.get("events")
        actor_email = deep_get(event, "actor", "email", default="EMAIL_UNKNOWN")
        if application_name == "drive" and events and ("acl_change" in set(e["type"] for e in events)):
            # If any of the events in this record are a dangerous file share, alert:
            return any(self._check_acl_change_event(actor_email, acl_change_event) for acl_change_event in events)
        return False

    def title(self, event):
        events = event.get("events", [])
        actor_email = deep_get(event, "actor", "email", default="EMAIL_UNKNOWN")
        matching_events = [
            self._check_acl_change_event(actor_email, acl_change_event)
            for acl_change_event in events
            if self._check_acl_change_event(actor_email, acl_change_event)
        ]
        if matching_events:
            len_events = len(matching_events)
            first_event = matching_events[0]
            actor = first_event.get("actor", "ACTOR_UNKNOWN")
            doc_title = first_event.get("doc_title", "DOC_TITLE_UNKNOWN")
            target_user = first_event.get("target_user", "USER_UNKNOWN")
            if len(matching_events) > 1:
                return (
                    f"Multiple dangerous shares ({len_events}) by [{actor}], including "
                    + f'"{doc_title}" to {target_user}'
                )
            return f'Dangerous file share by [{actor}]: "{doc_title}" to {target_user}'
        return "No matching events, but DangerousShares still fired"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context

slack_audit_logs_user_privilege_escalation_tests: list[RuleTest] = [
    RuleTest(
        name="Owner Transferred",
        expected_result=True,
        log={
            "action": "owner_transferred",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
        },
    ),
    RuleTest(
        name="Permissions Assigned",
        expected_result=True,
        log={
            "action": "permissions_assigned",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
        },
    ),
    RuleTest(
        name="Role Changed to Admin",
        expected_result=True,
        log={
            "action": "role_change_to_admin",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
        },
    ),
    RuleTest(
        name="Role Changed to Owner",
        expected_result=True,
        log={
            "action": "role_change_to_owner",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
        },
    ),
    RuleTest(
        name="User Logout",
        expected_result=False,
        log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


@panther_managed
class SlackAuditLogsUserPrivilegeEscalation(Rule):
    id = "Slack.AuditLogs.UserPrivilegeEscalation-prototype"
    display_name = "Slack User Privilege Escalation"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Privilege Escalation", "Account Manipulation", "Additional Cloud Roles"]
    reports = {"MITRE ATT&CK": ["TA0004:T1098.003"]}
    default_severity = Severity.HIGH
    default_description = "Detects when a Slack user gains escalated privileges"
    default_reference = "https://slack.com/intl/en-gb/help/articles/201314026-Permissions-by-role-in-Slack"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_user_privilege_escalation_tests
    USER_PRIV_ESC_ACTIONS = {
        "owner_transferred": "Slack Owner Transferred",
        "permissions_assigned": "Slack User Assigned Permissions",
        "role_change_to_admin": "Slack User Made Admin",
        "role_change_to_owner": "Slack User Made Owner",
    }

    def rule(self, event):
        return event.get("action") in self.USER_PRIV_ESC_ACTIONS

    def title(self, event):
        # This is the user taking the action.
        actor_username = deep_get(event, "actor", "user", "name", default="<unknown-actor>")
        actor_email = deep_get(event, "actor", "user", "email", default="<unknown-email>")
        # This is the user the action is taken on.
        entity_username = deep_get(event, "entity", "user", "name", default="<unknown-actor>")
        entity_email = deep_get(event, "entity", "user", "email", default="<unknown-email>")
        action = event.get("action")
        if action == "owner_transferred":
            return f"{self.USER_PRIV_ESC_ACTIONS[action]} from {actor_username} ({actor_email})"
        if action == "permissions_assigned":
            return f"{self.USER_PRIV_ESC_ACTIONS[action]} {entity_username} ({entity_email})"
        if action == "role_change_to_admin":
            return f"{self.USER_PRIV_ESC_ACTIONS[action]} {entity_username} ({entity_email})"
        if action == "role_change_to_owner":
            return f"{self.USER_PRIV_ESC_ACTIONS[action]} {entity_username} ({entity_email})"
        return f"Slack User Privilege Escalation event {action} on {entity_username} ({entity_email})"

    def severity(self, event):
        # Downgrade severity for users assigned permissions
        if event.get("action") == "permissions_assigned":
            return "Medium"
        return "Critical"

    def alert_context(self, event):
        return slack_alert_context(event)



"""Global helpers for Azure SignIn detections"""

from pypanther.helpers.panther_base_helpers import deep_get


def actor_user(event):
    category = deep_get(event, "category", default="")
    if category in {"ServicePrincipalSignInLogs"}:
        return deep_get(event, "properties", "servicePrincipalName")
    if category in {"SignInLogs", "NonInteractiveUserSignInLogs"}:
        return deep_get(event, "properties", "userPrincipalName")
    return None


def is_sign_in_event(event):
    return deep_get(event, "operationName", default="") == "Sign-in activity"


def azure_signin_alert_context(event) -> dict:
    ac_actor_user = actor_user(event)
    if ac_actor_user is None:
        ac_actor_user = "<NO_ACTORUSER>"
    a_c = {}
    a_c["tenantId"] = deep_get(event, "tenantId", default="<NO_TENANTID>")
    a_c["source_ip"] = deep_get(event, "properties", "ipAddress", default="<NO_SOURCEIP>")
    a_c["actor_user"] = ac_actor_user
    a_c["resourceDisplayName"] = deep_get(
        event,
        "properties",
        "resourceDisplayName",
        default="<NO_RESOURCEDISPLAYNAME>",
    )
    a_c["resourceId"] = deep_get(event, "properties", "resourceId", default="<NO_RESOURCEID>")
    return a_c



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_rule_tests: list[RuleTest] = [
    RuleTest(
        name="Non Triggered Rule",
        expected_result=False,
        log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {"severity": "HIGH", "triggered_actions": None},
        },
    ),
    RuleTest(
        name="High Severity Rule",
        expected_result=True,
        log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "data_source": "DRIVE",
                "severity": "HIGH",
                "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
            },
        },
    ),
    RuleTest(
        name="Medium Severity Rule",
        expected_result=True,
        log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "data_source": "DRIVE",
                "severity": "MEDIUM",
                "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
            },
        },
    ),
    RuleTest(
        name="Low Severity Rule",
        expected_result=True,
        log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {"severity": "LOW", "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}]},
        },
    ),
    RuleTest(
        name="High Severity Rule with Rule Name",
        expected_result=True,
        log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "severity": "HIGH",
                "rule_name": "CEO Impersonation",
                "triggered_actions": [{"action_type": "MAIL_MARK_AS_PHISHING"}],
            },
        },
    ),
]


@panther_managed
class GSuiteRule(Rule):
    id = "GSuite.Rule-prototype"
    display_name = "GSuite Passthrough Rule Triggered"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.INFO
    default_description = "A GSuite rule was triggered.\n"
    default_reference = "https://support.google.com/a/answer/9420866"
    default_runbook = "Investigate what triggered the rule.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_rule_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "rules":
            return False
        if not deep_get(event, "parameters", "triggered_actions"):
            return False
        return True

    def title(self, event):
        rule_severity = deep_get(event, "parameters", "severity")
        if deep_get(event, "parameters", "rule_name"):
            return "GSuite " + rule_severity + " Severity Rule Triggered: " + deep_get(event, "parameters", "rule_name")
        return "GSuite " + rule_severity + " Severity Rule Triggered"

    def severity(self, event):
        return deep_get(event, "parameters", "severity", default="INFO")



from datetime import timedelta

from panther_detection_helpers.caching import put_string_set

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_oss_helpers import resolve_timestamp_string

standard_new_user_account_created_tests: list[RuleTest] = [
    RuleTest(
        name="User Creation Event - OneLogin",
        expected_result=True,
        mocks=[RuleMock(object_name="put_string_set", return_value="")],
        log={
            "event_type_id": 13,
            "actor_user_id": 123456,
            "user_id": 12345,
            "actor_user_name": "Bob Cat",
            "user_name": "Bob Cat",
            "p_event_time": "2021-06-27 00:08:28.792Z",
            "p_log_type": "OneLogin.Events",
            "p_row_id": "aaaaaaaabbbbbbbbbbbbccccccccc",
        },
    ),
    RuleTest(
        name="Standard Login Event - OneLogin",
        expected_result=False,
        log={
            "event_type_id": 5,
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_name": "Bob Cat",
            "user_id": 12345,
            "ipaddr": "192.168.1.1",
            "p_event_time": "2021-06-27 00:08:28.792Z",
            "p_log_type": "OneLogin.Events",
            "p_row_id": "aaaaaaaabbbbbbbbbbbbccccccccc",
        },
    ),
    RuleTest(
        name="User Account Created - CloudTrail",
        expected_result=True,
        mocks=[RuleMock(object_name="put_string_set", return_value="")],
        log={
            "eventName": "CreateUser",
            "responseElements": {"user": {"userName": "Bob Cat", "userId": "12345"}},
            "p_event_time": "2021-08-31 15:46:02.000000000",
            "p_log_type": "AWS.CloudTrail",
            "p_row_id": "aaaaaaaabbbbbbbbbbbbccccccccc",
        },
    ),
    RuleTest(
        name="Normal Console Login - CloudTrail",
        expected_result=False,
        log={
            "userIdentity": {"type": "IAMUser", "userName": "some_user"},
            "eventName": "ConsoleLogin",
            "responseElements": {"ConsoleLogin": "Success"},
            "p_event_time": "2021-06-04 09:59:53.650807",
            "p_row_id": "aaaaaaaabbbbbbbbbbbbccccccccc",
            "p_log_type": "AWS.CloudTrail",
        },
    ),
    RuleTest(
        name="User Creation Event - Zoom",
        expected_result=True,
        mocks=[RuleMock(object_name="put_string_set", return_value="")],
        log={
            "action": "Add",
            "category_type": "User",
            "operation_detail": "Add User homer@simpson.io  - User Type: Basic - Department: Foo",
            "operator": "abe@simpson.io",
            "p_log_type": "Zoom.Operation",
            "p_event_time": "2021-06-27 00:08:28.792Z",
        },
    ),
]


@panther_managed
class StandardNewUserAccountCreated(Rule):
    id = "Standard.NewUserAccountCreated-prototype"
    display_name = "New User Account Created"
    log_types = [LogType.ONELOGIN_EVENTS, LogType.AWS_CLOUDTRAIL, LogType.ZOOM_OPERATION]
    tags = ["DataModel", "Indicator Collection", "OneLogin", "Persistence:Create Account"]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_description = "A new account was created"
    default_runbook = (
        "A new user account was created, ensure it was created through standard practice and is for a valid purpose."
    )
    default_reference = "https://attack.mitre.org/techniques/T1136/001/"
    summary_attributes = ["p_any_usernames"]
    tests = standard_new_user_account_created_tests
    # Days an account is considered new
    TTL = timedelta(days=3)

    def rule(self, event):
        if event.udm("event_type") != event_type.USER_ACCOUNT_CREATED:
            return False
        user_event_id = f"new_user_{event.get('p_row_id')}"
        new_user = event.udm("user") or "<UNKNOWN_USER>"
        new_account = event.udm("user_account_id") or "<UNKNOWN_ACCOUNT>"
        event_time = resolve_timestamp_string(event.get("p_event_time"))
        expiry_time = event_time + self.TTL
        if new_user:
            put_string_set(new_user + "-" + str(new_account), [user_event_id], expiry_time.strftime("%s"))
        return True

    def title(self, event):
        return f"A new user account was created - [{event.udm('user') or '<UNKNOWN_USER>'}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import pattern_match_list

teleport_create_user_accounts_tests: list[RuleTest] = [
    RuleTest(
        name="Echo command",
        expected_result=False,
        log={
            "argv": [],
            "cgroup_id": 4294967537,
            "code": "T4000I",
            "ei": 15,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/echo",
            "pid": 7143,
            "ppid": 7115,
            "program": "echo",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
            "time": "2020-08-17T17:40:37.491Z",
            "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
            "user": "panther",
        },
    ),
    RuleTest(
        name="Userdel command",
        expected_result=True,
        log={
            "argv": ["jacknew"],
            "cgroup_id": 4294967567,
            "code": "T4000I",
            "ei": 105,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/sbin/userdel",
            "pid": 8931,
            "ppid": 8930,
            "program": "userdel",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "4244c271-8069-4679-a27e-f7c18f88ce45",
            "time": "2020-08-17T18:39:26.192Z",
            "uid": "346d3f61-a010-4871-84de-897f50b18118",
            "user": "panther",
        },
    ),
]


@panther_managed
class TeleportCreateUserAccounts(Rule):
    id = "Teleport.CreateUserAccounts-prototype"
    display_name = "Teleport Create User Accounts"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["SSH", "Persistence:Create Account"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_severity = Severity.HIGH
    default_description = "A user has been manually created, modified, or deleted"
    dedup_period_minutes = 15
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Analyze why it was manually created and delete it if necessary."
    summary_attributes = ["event", "code", "user", "program", "path", "return_code", "login", "server_id", "sid"]
    tests = teleport_create_user_accounts_tests  # user password expiry
    # change passwords for users
    # create, modify, and delete users
    USER_CREATE_PATTERNS = ["chage", "passwd", "user*"]

    def rule(self, event):
        # Filter the events
        if event.get("event") != "session.command":
            return False
        # Check that the program matches our list above
        return pattern_match_list(event.get("program", ""), self.USER_CREATE_PATTERNS)

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] has manually modified system users on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"
